@@ -1,5 +1,4 @@
 """
-===============================================================
 Contact mechanics sub models (:mod:`slippy.contact.sub_models`)
 ===============================================================
 
@@ -7,12 +6,7 @@ Contact mechanics sub models (:mod:`slippy.contact.sub_models`)
 
 In slippy, things that are not solved in the main normal contact loop are dealt with using sub models. These could be
 anything from flash temperature to tribofilm growth to friction etc.. These sub models cover behaviour that doesn't
-the normal contact problem or acts on a slower time scale. For example: wear of the surfaces does affect the normal
-contact problem but over a very slow time scale (compared to the reaction of the material) which mean that wear should
-be solved as a sub model. Likewise, in many situations the friction force can be solved separately to the normal contact
-problem, thus friction can also be a sub model. However, when solving a lubrication model for example the pressure-
-viscosity response of the lubricant strongly affects the normal contact problem and acts in the same time scale to the
-material response, thus this should be solved as part of the main contact loop and is not a valid sub model.
+need to be strongly coupled to the normal contact problem or acts on a slower time scale.
 
 Currently implemented sub models are:
 
@@ -29,6 +23,12 @@ from .epp_wear import WearElasticPerfectlyPlastic
 from .friction_coulomb_model import FrictionCoulombSimple
 from .tangential_pure_sliding import TangentialPureSliding
 from .tangential_partial_slip import TangentialPartialSlip
+from .sub_surface_stress import SubsurfaceStress
+from .rigid_body_displacement import RollingSliding1D, RigidBodyDisplacementSliding
+from .dummy_value import DummyValue
+from .fill_displacements import FillDisplacements
+from ._TransientSubModelABC import _TransientSubModelABC
 
 __all__ = ['ResultContactStiffness', 'WearElasticPerfectlyPlastic', 'FrictionCoulombSimple', 'TangentialPureSliding',
-           'TangentialPartialSlip']
+           'TangentialPartialSlip', 'SubsurfaceStress', 'RigidBodyDisplacementSliding',
+           'RollingSliding1D', 'DummyValue', '_TransientSubModelABC', 'FillDisplacements']
